/*****************************************************************************/
/* FILE NAME: enettest.c                        COPYRIGHT (c) Freescale 2009 */

/*                                                       All Rights Reserved */
/*****************************************************************************/
#include "string.h"

#include "MPC5675K.h"
//#include "mpc5675k-2.04.h"
#include "typedefs.h"
#include "nbuf.h"
#include "serial.h"

//#define NULL	0
#define FAIL	0
#define PASS	1
#define FALSE	0
#define TRUE	1
#define TRUE_REJ 2
#define BROADCAST 2
#define READ	0x60020000
#define WRITE	0x50020000
#define TIMEOUT 700000

/*************************** Function Declarations ******************************/
unsigned short ethernet_test(uint8_t mismatch_addr, uint8_t promiscuous_mode, uint8_t broadcast, uint8_t hash, uint8_t *addr, uint16_t *status);
uint8_t fec_hash_address(uint8_t *addr);
void MSCR_Init(uint8_t preamble, uint8_t speed);
uint16_t Management_Read(uint8_t phy_addr, uint8_t reg_addr);
void Management_Write(uint8_t phy_addr, uint8_t reg_addr, uint16_t data);
void PHY_Init(void);
void SIU_PCR_Init(void);
void stop(void);
void Test1(void);
void Test2(void);
void Test3(void);
void Test4(void);
void Test5(void);
void Test6(void);
void TimeOut(void);
void enettest(void);

/*************************** Global Variables ******************************/
//diab
//#pragma section BD ".bd" ".bd"	
//#pragma use_section BD txbd, rxbd
//GHS
//#pragma ghs section bss=".bd"
//uint8_t txbd[sizeof(NBUF) * (TX_BUFFER_SIZE * NUM_TXBDS)];
//uint8_t rxbd[sizeof(NBUF) * (TX_BUFFER_SIZE * NUM_RXBDS)];

//#pragma section bss=default
//#pragma section bss=".bd"
uint8_t txbd[sizeof(NBUF) * (TX_BUFFER_SIZE * NUM_TXBDS)];
uint8_t rxbd[sizeof(NBUF) * (TX_BUFFER_SIZE * NUM_RXBDS)];
//#pragma section bss=default


//diab
//#pragma section BUF ".buf" ".buf"
//#pragma use_section BUF TxBuffer, RxBuffer
//#pragma ghs section bss=".buf"
uint8_t TxBuffer[TX_BUFFER_SIZE * NUM_TXBDS];
uint8_t RxBuffer[RX_BUFFER_SIZE * NUM_RXBDS];
//#pragma ghs section bss=default

uint16_t reg_data1[0x20], reg_data2[0x20];
uint16_t fail_1[100], fail_2[100], fail_3[100], fail_4[100], fail_5[100], fail_6[100];
uint16_t fail_total = 0;
uint16_t debug_temp;
uint16_t test_num, timeout_count = 0, bad_data_count = 0;

/********************************** main *************************************/
void enettest(void) {

	uint16_t data;

	SIU_PCR_Init();
	PHY_Init();

	test_num = 1;
	Test1();
/*	test_num = 2;
 - 	Test2();
 - 	test_num = 3;
 - 	Test3();
 - 	test_num = 4;
 - 	Test4();
 - 	test_num = 5;
 - 	Test5();
 - 	test_num = 6;
 - 	Test6();*/


	//reset PHY
	Management_Write(1, 0, 0x8000);

	//wait for reset to complete
//	while(data = Management_Read(1, 0) & 0x8000){;}
	do{
		data = Management_Read(1, 0) & 0x8000;
	}while(data);
	


}/* end of enettest() */


/*********************************************************************************************
FUNCTION        : ethernet_test 
INPUTS NOTES    : 
RETURNS NOTES   : PASS or FAIL.
GENERAL NOTES   : Receives a transmission in loop-back mode.                
**********************************************************************************************/
unsigned short ethernet_test(uint8_t mismatch_addr, uint8_t promiscuous_mode, uint8_t broadcast, uint8_t hash, uint8_t *addr, uint16_t *status)
{
	uint32_t i;
	NBUF *pTxNbuf, *pRxNbuf, *TxNBUF, *RxNBUF;
	
	uint32_t fail = 0;

	/* Ethernet data to be transmitted */
	uint8_t packet[] = 
	{
		0x00, 0xCF, 0x52, 0x82, 0xC3, 0x01,   //destination address
		0x00, 0xCF, 0x52, 0x82, 0xC3, 0x01,   //source address
		0x08, 0x00,							  //type (IP in this case)
		0x45, 0x00,	0x00, 0x3C, 0x2B, 0xE8, 0x00, 0x00, //data
		0x20, 0x01, 0xA6, 0x1B, 0xA3, 0x0A, 0x41, 0x55, 
		0xA3, 0x0A,	0x41, 0x54, 0x08, 0x00, 0x0C, 0x5C,
		0x01, 0x00, 0x40, 0x00, 0x61, 0x62, 0x63, 0x64,
		0x65, 0x66,	0x67, 0x68, 0x69, 0x6A, 0x6B, 0x6C,
		0x6D, 0x6E, 0x6F, 0x70, 0x71, 0x72, 0x73, 0x74,
		0x75, 0x76
	};
	
	if(broadcast == TRUE)
	{
		for(i = 0; i < 6; i++)  //DA = FF:FF:FF:FF:FF:FF
			packet[i] = 0xFF;
	}
	else if(hash != NULL)
	{
		for(i = 0; i < 6; i++)	
			packet[i] = addr[i];
	}
	

	TxNBUF = (NBUF *)txbd;
	RxNBUF = (NBUF *)rxbd;
	/* reset the FEC - clears EIR, etc. */
	FEC.ECR.B.RESET = 1;  

	/* Initialize receive descriptor ring */
	for (i = 0; i < NUM_RXBDS; i++)
	{
		RxNBUF[i].status = RX_BD_E;
		RxNBUF[i].length = 0;
		RxNBUF[i].data = &RxBuffer[i * RX_BUFFER_SIZE];
	}
	/* Set the Wrap bit on the last one in the ring */
	RxNBUF[NUM_RXBDS - 1].status |= RX_BD_W;

	/* Initialize transmit descriptor ring */
	for (i = 0; i < NUM_TXBDS; i++)
	{
		TxNBUF[i].status = TX_BD_L | TX_BD_TC;
		TxNBUF[i].length = 0;
		TxNBUF[i].data = &TxBuffer[i * TX_BUFFER_SIZE];
	}
	/* Set the Wrap bit on the last one in the ring */
	TxNBUF[NUM_TXBDS - 1].status |= TX_BD_W;
	
	/* Set the source address for the controller */
	if(mismatch_addr == FALSE)
	{
		FEC.PALR.R = 0x00CF5282;  //set to match DA in packet
		FEC.PAUR.R = 0xC3010000;
	}
	else if(mismatch_addr == TRUE)
	{
		FEC.PALR.R = 0x00BABECA;  //set to not match DA in packet and cause failure
		FEC.PAUR.R = 0xFE010000;
	}
	
	//initially no hash matching
	FEC.IALR.R = 0x00000000; //no individual hash matching
	FEC.IAUR.R = 0x00000000;
	FEC.GALR.R = 0x00000000; //no group hash matching
	FEC.GAUR.R = 0x00000000;

	
	if (hash != NULL)
	{
		if(addr[0] & 0x1)
		{//group address
			if(hash >= 32)
				FEC.GAUR.R = (uint32_t)(1 << (hash - 32));
			else
				FEC.GALR.R = (uint32_t)(1 << hash);					
		}
		else
		{//individual address
			if(hash >= 32)
				FEC.IAUR.R = (uint32_t)(1 << (hash - 32));
			else
				FEC.IALR.R = (uint32_t)(1 << hash);			
		}
	}
		
	/* Set Receive Buffer Size */
	FEC.EMRBR.R = (uint16_t)RX_BUFFER_SIZE;

	/* Point to the start of the circular Rx buffer descriptor queue */
	FEC.ERDSR.R = (uint32_t)RxNBUF;

	/* Point to the start of the circular Tx buffer descriptor queue */
	FEC.ETDSR.R = (uint32_t)TxNBUF;

	if(!promiscuous_mode)
		/* Set the tranceiver interface to MII mode */
		FEC.RCR.R = ((64+18)<<16 | MPC_FEC_RCR_MII_MODE);    /* MII mode - 18 signals;  MAX_FL = 64bytes data + 18(for DA, SA, and CRC)*/
	else
		/* Set the tranceiver interface to MII mode and promiscuous mode */
		FEC.RCR.R = ((64+18)<<16 | MPC_FEC_RCR_MII_MODE | MPC_FEC_RCR_PROM);    /* MII mode - 18 signals;  MAX_FL = 64bytes data + 18(for DA, SA, and CRC)*/

	/* Grab buffer in ring */
	pTxNbuf = TxNBUF;
	pRxNbuf = RxNBUF;
	
	/* Operate in full-duplex mode, no heart beat control */
	FEC.TCR.R = 0x0004;
	
	/* Set MII speed to be 2.67Mhz */
	FEC.MSCR.R = 0x0018;
	

	
	/* Copy constant data into the data buffer */
	memcpy(pTxNbuf->data, packet, 64);
		
	/* Set the length of the packet */
	pTxNbuf->length = 64;
	
	/* Enable FEC */
	FEC.ECR.B.ETHER_EN = 1;

	/* Indicate Empty buffers have been produced */
	FEC.RDAR.B.R_DES_ACTIVE = 1;
		
	/* Mark packet as ready to send */ 
	pTxNbuf->status |= TX_BD_R;
	
	//FEC.MSCR.R = 0x0040;
	//debug_temp = Management_Read(1, 0); //debug
	
	/* Indicate to FEC that transmit buffer is ready to send */
	FEC.TDAR.B.X_DES_ACTIVE = 1;

	for (i=0; i < TIMEOUT; i++)
	{
		if (FEC.EIR.B.RXF) 
		{
			break;		
		}
	}
	
	if (i == TIMEOUT)
	{
		/* Timed-out */
		fail++;
		timeout_count++;
		//TimeOut();
	}	
	
	for (i = 0; i < 64; i++)
	{
		if (TxNBUF[0].data[i] != RxNBUF[0].data[i])
		{
			fail++;
			bad_data_count++;
		}
	}	
	
	*status = pRxNbuf->status;

	if (fail)
		return (FAIL);
	else
		return (PASS);
	
}   /* end of ethernet_test */



/********************************************************************/
/*
 * Generate the hash table settings for the given address
 *
 * Parameters:
 *  addr    48-bit (6 byte) Address to generate the hash for
 *
 * Return Value:
 *  The 6 most significant bits of the 32-bit CRC result
 */
uint8_t fec_hash_address(uint8_t *addr)
{
    uint32_t crc;
    uint8_t byte;
    int i, j;

    crc = 0xFFFFFFFF;
    for(i=0; i<6; ++i)
    {
        byte = addr[i];
        for(j=0; j<8; ++j)
        {
            if((byte & 0x01)^(crc & 0x01))
            {
                crc >>= 1;
                crc = crc ^ 0xEDB88320;
            }
            else
                crc >>= 1;
            byte >>= 1;
        }
    }
    return (uint8_t)(crc >> 26);
}


void MSCR_Init(uint8_t preamble, uint8_t speed)
{
	FEC.MSCR.R = (uint32_t)(0 | (preamble << 7) | (speed << 1));
}

uint16_t Management_Read(uint8_t phy_addr, uint8_t reg_addr)
{
	FEC.EIR.B.MII = 0x1;

	FEC.MMFR.R = (uint32_t)(READ | (phy_addr << 23) | (reg_addr << 18));

	while(!FEC.EIR.B.MII){}

	FEC.EIR.B.MII = 0x1;

	return (uint16_t)(FEC.MMFR.R & 0x0000FFFF);
}

void Management_Write(uint8_t phy_addr, uint8_t reg_addr, uint16_t data)
{
	FEC.EIR.B.MII = 0x1;

	FEC.MMFR.R = (uint32_t)(WRITE | (phy_addr << 23) | (reg_addr << 18) | data);

	while(!FEC.EIR.B.MII){}

	FEC.EIR.B.MII = 0x1;

}

void SIU_PCR_Init(void){

    SIU.PCR[198].R = 0x0706;      // Set to FEC_MDIO	  
    SIU.PCR[199].R = 0x0604;      // Set to FEC_MDC 	  
    SIU.PCR[200].R = 0x0604;      // Set to FEC_TX_EN 
    SIU.PCR[201].R = 0x0604;      // Set to FEC_TXD[0]	  
    SIU.PCR[202].R = 0x0604;      // Set to FEC_TXD[1]  
    SIU.PCR[203].R = 0x0604;      // Set to FEC_TXD[2]
    SIU.PCR[204].R = 0x0604;      // Set to FEC_TXD[3]  
    //SIU.PCR[205].R = 0x0604;      // Set to FEC_TX_ER   -
    SIU.PCR[206].R = 0x0506;      // Set to FEC_COL	 
    SIU.PCR[207].R = 0x0506;      // Set to FEC_TX_CLK  
    SIU.PCR[208].R = 0x0506;      // Set to FEC_CRS	 
    SIU.PCR[209].R = 0x0506;      // Set to FEC_RX_CLK 
    SIU.PCR[210].R = 0x0506;      // Set to FEC_RX_DV  
    SIU.PCR[211].R = 0x0506;      // Set to FEC_RXD[0]   
    SIU.PCR[212].R = 0x0506;      // Set to FEC_RXD[1] 
    SIU.PCR[213].R = 0x0506;      // Set to FEC_RXD[2] 
    SIU.PCR[214].R = 0x0506;      // Set to FEC_RXD[3] 
    SIU.PCR[215].R = 0x0506;      // Set to FEC_RX_ER    
}

void Test1(void)
{
	uint16_t i, status;
	uint8_t hash;
	uint8_t addr[6];

	timeout_count = 0; bad_data_count = 0;
	//test 1 - PA match
	hash = NULL;
	for (i = 0;i < 100; i++ )
	{
		fail_1[i] = 0;
		if(ethernet_test(FALSE, FALSE, FALSE, hash, &addr[0], &status) == FAIL)
		{//don't cause intential failure
			fail_1[i] = 1;
			fail_total++;
		}
		if(status != (RX_BD_L | RX_BD_W))  //last buffer for packet and wrap
		{
			fail_1[i] = 1;
			fail_total++;
		}
	}	

	
      if (fail_total << 3){
	  printf("\nEthernet test Passed");	
      }
      else {
	  printf("\nFAILED: Check ethernet circuit and re-run test");
      }

}

void Test2(void)
{
	uint16_t i, status;
	uint8_t hash;
	uint8_t addr[6];

	timeout_count = 0; bad_data_count = 0;
	//test 2 - PA mismatch
	hash = NULL;
	for (i = 0;i < 100; i++ )
	{
		fail_2[i] = 0;
		if(ethernet_test(TRUE, FALSE, FALSE, hash, &addr[0], &status) == PASS)  //cause failure by setting PA to bad value
		{
			fail_2[i] = 1;
			fail_total++;
		}
		if(status != (RX_BD_W | RX_BD_E))  //buffer still empty and wrap
		{
			fail_2[i] = 1;
			fail_total++;
		}
	}
}

void Test3(void)
{
	uint16_t i, status;
	uint8_t hash;
	uint8_t addr[6];

	timeout_count = 0; bad_data_count = 0;
	//test 3 - promiscuous mode
	hash = NULL;
	for (i = 0;i < 100; i++ )
	{
		fail_3[i] = 0;
		if(ethernet_test(TRUE, TRUE, FALSE, hash, &addr[0], &status) == FAIL)  //mismatch addr, but we should receive packets due to promiscuous mode
		{
			fail_3[i] = 1;
			fail_total++;
		}
		if(status != (RX_BD_W | RX_BD_L | RX_BD_M)) //last buffer for packet and miss bit set and wrap
		{
			fail_3[i] = 1;
			fail_total++;
		}
	}
}

void Test4(void)
{
	uint16_t i, status;
	uint8_t hash;
	uint8_t addr[6];

	timeout_count = 0; bad_data_count = 0;
	//test 4 - broacast
	hash = NULL;
	for (i = 0;i < 100; i++ )
	{
		fail_4[i] = 0;
		if(ethernet_test(FALSE, FALSE, TRUE, hash, &addr[0], &status) == FAIL)  //broadcast addr, we should receive packets
		{
			fail_4[i] = 1;
			fail_total++;
		}
		if(status != (RX_BD_L | RX_BD_W | RX_BD_BC)) //last buffer and broadcast and wrap
		{
			fail_4[i] = 1;
			fail_total++;
		}
	}
}

void Test5(void)
{
	uint16_t i, status;
	uint8_t hash;
	uint8_t addr[6];

	timeout_count = 0; bad_data_count = 0;
	//test 5 - individual hash match
	addr[0] = 0x06; addr[1] = 0xFF; addr[2] = 0xFF; addr[3] = 0xFF; addr[4] = 0xFF; addr[5] = 0xFF;
	hash = fec_hash_address(&addr[0]);  //corresponding with DA 06:FF:FF:FF:FF:FF
	for (i = 0;i < 100; i++ )
	{
		fail_5[i] = 0;
		if(ethernet_test(FALSE, FALSE, FALSE, hash, &addr[0], &status) == FAIL)  //don't cause intential failure
		{
			fail_5[i] = 1;
			fail_total++;
		}
		if(status != (RX_BD_L | RX_BD_W))  //last buffer for packet and wrap
		{
			fail_5[i] = 1;
			fail_total++;
		}
	}
}

void Test6(void)
{
	uint16_t i, status;
	uint8_t hash;
	uint8_t addr[6];

	timeout_count = 0; bad_data_count = 0;
	//test 6 - group hash match
	addr[0] = 0x07; addr[1] = 0xFF; addr[2] = 0xFF; addr[3] = 0xFF; addr[4] = 0xFF; addr[5] = 0xFF;
	hash = fec_hash_address(&addr[0]);  //corresponding with DA 07:FF:FF:FF:FF:FF
	for (i = 0;i < 100; i++ )
	{
		fail_6[i] = 0;
		if(ethernet_test(FALSE, FALSE, FALSE, hash, &addr[0], &status) == FAIL)  //don't cause intential failure
		{
			fail_6[i] = 1;
			fail_total++;
		}
		if(status != (RX_BD_L | RX_BD_W | RX_BD_MC))  //last buffer for packet, wrap, and multicast
		{
			fail_6[i] = 1;
			fail_total++;
		}
	}
}

void stop(void)
{
	while(1){;}
}

void TimeOut(void)
{
	return;
}

void PHY_Init(void)
{
	uint16_t data;
	int i;

	/* Set MII speed */
	FEC.MSCR.R = 0x0040;

	for(i = 0; i < 32; i++)
	{
		data = Management_Read(1, (uint8_t)i);
	}

	//reset PHY
	Management_Write(1, 0, 0x8000);

	//wait for reset to complete
//	while(data = Management_Read(1, 0) & 0x8000){;}
	do{
		data = Management_Read(1, 0) & 0x8000;
	}while(data);
	
	//auto-negotiation off
	data = Management_Read(1, 0);
	data &= 0xEFFF; //clear auto negotiate bit
	Management_Write(1, 0, data);
	data = Management_Read(1, 0);
	
	//loopback enabled 
	data = Management_Read(1, 0);
	data |= 0x4000; //set loopback Bit
	Management_Write(1, 0, data);
	data = Management_Read(1, 0);
	//Management_Write(1, 0, 0x7100);
	
	// check that we are in loopback mode
	//while(!(data = Management_Read(1, 16) & 0x0008)){;}
//	do{
//		data = Management_Read(1, 16) & 0x0008;
//	}while(!data);
	
	//data = Management_Read(1, 0);
	
	//change to 10MB
	//data = Management_Read(1, 0);
	//data &= 0xDFFF; //clear Speed Select Bit
	//Management_Write(1, 0, data);
	//data = Management_Read(1, 0);
	
}

