#include "MPC5675K.h"

#include "common.h"
#include "serial.h"

extern void __start_p1();	// Prototype for second core startup

void DISABLE_WATCHDOG();
void initialize_hardware();
void setup_fmpll();
void CLKOUT();
void setup_EBI();
void SetBusMode(char mode);

void delay(int count);
uint8_t GetIdPort();
char TestPrompt(char p);
void GPIOTest(char p, uint8_t *PNum);

void FlexrayOpTest();
void ExpansionConTest();
void CANTest();
void PushSwitchTest();
void LANTest();

char LEDDelay();
char TogDelay();
void LEDTest();

void EBIWriteTest();
void DDRTest();
void enettest();


uint8_t   error;

// --------------------------------------------------------------------------
int main(void) {
  volatile int i = 0;
  char ch;

	DISABLE_WATCHDOG();				// Disable watchdog 
	RGM.FES.B.F_FCCU_SAFE = 0x1;  	// Clear any fails Debug ONLY
  
	initialize_hardware();
  
#if ROM_VERSION == 1
/* Start the second core, VLE mode*/
SSCM.DPMBOOT.R = (unsigned long)__start_p1 + 0x00000002;
SSCM.DPMKEY.R = 0x00005AF0;
SSCM.DPMKEY.R = 0x0000A50F;
#endif

	// main loop
	for (;;) {
		printf("\n\nAXM-0591 (CMM-5675) Diagnostic Program  --  %s\n\n", __DATE__);

		printf("  1 Expansion Connection\n");
		printf("  2 Flexray\n");
		printf("  3 CAN\n");
		printf("  4 Push Switch\n");
		printf("  5 LEDs\n");
		printf("  6 LAN\n");
		printf("  7 EBI - Write Test\n");
		printf("  9 DDR Memory\n");
		printf("\n  0 Test All\n");

		printf("\n>"); // prompt
		
		ch = getchar();	// wait for a character from serial port
		putchar(ch);        // echo it
		
		switch (ch){
			case '1': ExpansionConTest(); break;
			case '2': FlexrayOpTest();  break;
			case '3': CANTest();  break;
			case '4': PushSwitchTest(); break;
			case '5': LEDTest();  break;
			case '6': LANTest();  break;
			case '7': EBIWriteTest();  break;
			case '9': DDRTest();  break;
			case '0': 
			  ExpansionConTest();
			  FlexrayOpTest();
			  CANTest();
			  PushSwitchTest();
			  LEDTest();
              LANTest();
              EBIWriteTest();
			  DDRTest();
			  break;
			default:	printf("\n\nUnknown Command");					break;
		}
		putchar('\n');
	}
}


static const char *TestNames[] = {
  "",
  "Flexray Optical Module",
  "LED's",
  "Expansion Connection",
  "CAN A & B",
  0
};
static const char *TestPBGA[] = {
  "",
  "SCI2: TXD  RXD\n   FRO_IO: 0-7\n   FR_A: BGE  EN  TX  TXEN\n   FR_B: BGE  EN  TX  TXEN\n   DSPI_A: SCK  SIN  SOUT  CS0",
  "LED_1",
  "SCI_B: TXD  RXD\n   DSPI_C: SCK  SIN  SOUT  CS0  CS1\n   FP0A0-2, FP1X1-3, FP1A1, FP1A3, FP1B1, FP1B3, FPX02, FPX03",
  "CAN_A: TX  RS\n   CAN_B: TX  RS",
  0
};

 
char TestPrompt(char p){
  puts("\n\n\n------ ");
  puts((char *)TestNames[p]);
  puts("\n\nVerify these are toggling:\n   ");
  puts((char *)TestPBGA[p]);
  puts("\n\npress any key to exit..");
}


// --------------------------------------------------------------------------
void GPIOTest(char p, uint8_t *PNum){
  uint8_t i;
  	
  // Enable pins as outputs
  for(i=0; *(PNum+i); i++){
	  SIU.PCR[*(PNum+i)].R = 0x0200;	
	}
   
  delay(10);  

  TestPrompt(p);

  // continuously toggle pins until user presses a key
  for( ; ;){
    for(i=0; *(PNum+i); i++)
      SIU.GPDO[*(PNum+i)].B.PDO = 1;   // make pin high
    if(TogDelay()) return; 

    for(i=0; *(PNum+i); i++)
      SIU.GPDO[*(PNum+i)].B.PDO = 0;   // make pin low
    if(TogDelay()) return; 
  }
}

// --------------------------------------------------------------------------
void CANTest(){
  uint8_t PNum[] = {16, 14, 139, 140, 0};
  GPIOTest(4, PNum);
}

// --------------------------------------------------------------------------
void FlexrayOpTest(){
  uint8_t PNum[] = {57, 60, 48, 47, 43, 134, 51, 52, 44, 137, 37, 39, 38, 36, 54, 53, 102, 59, 62, 100, 103, 116, 0};
  GPIOTest(1, PNum);

}

// --------------------------------------------------------------------------
void ExpansionConTest(){
  
  uint8_t PNum[] = {94, 95, 11, 13, 12, 10, 9, 58, 80, 99, 119, 122, 125, 120, 126, 121, 127, 98, 101, 0};
  GPIOTest(3, PNum);
}

#define LED1_GPIO	108
// --------------------------------------------------------------------------
void LEDTest(){
  
  SIU.PCR[LED1_GPIO].R = 0x0200;		// Enable pins as outputs
   
  delay(10);  

  TestPrompt(2);

	// LED's are active LOW

  // continuously toggle pins until user presses a key
  for( ; ;){
    SIU.GPDO[LED1_GPIO].B.PDO = 1;  // make pin high (off)
    if(LEDDelay()) break; 

    SIU.GPDO[LED1_GPIO].B.PDO = 0;  // make pin low  (on)
    if(LEDDelay()) break; 
  }
  SIU.GPDO[LED1_GPIO].B.PDO = 1;  // make pin high (off)
}

#define SW1_GPIO	12
// --------------------------------------------------------------------------
void PushSwitchTest(){
  
  puts("\n\n\n------ Push Switch");

  WKUP.NCR.B.NDSS = 3;	// disable NMI interrupts
  WKUP.NCR.B.NFEE = 1;	// Falling Edge Event enable
  //WKUP.NCR.B.NFE = 1;	// enable analog filter

  delay(10);  // must wait before reading
  if(WKUP.NSR.B.NIF == 1){
    puts("\n FAIL - SW2 button is held down");
  }else{
	  puts("\n\nPress SW2,   any key to exit..");
	  while(1){
		if(WKUP.NSR.B.NIF == 1){
			 puts(" PASSED - SW2 Pressed");
			break;
		}
	    if(kbhit()){  // return if user aborts
	      getchar();
	      break;
	    }
	  }
  }
  WKUP.NSR.B.NIF = 1;	// clear flag
  
  // Enable pins as inputs, push pull
  SIU.PCR[SW1_GPIO].R = 0x0103;	
  delay(10);  // must wait before reading

  if(SIU.GPDI[SW1_GPIO].B.PDIN == 0){
    puts("\n FAIL - SW1 button input is held down");
    return;
  }

  puts("\n\nPress SW1,   any key to exit..");
  while(SIU.GPDI[SW1_GPIO].B.PDIN == 1){
    if(kbhit()){  // return if user aborts
      getchar();
      return;
    }
  }
  puts(" PASSED - SW1 Pressed");
}

// --------------------------------------------------------------------------
void LANTest(){
 
  printf("\nTesting Ethernet Connection...");
 	enettest(); 
  
  printf("\n   Verify LAN LED's are ON then press a key to continue...");
  (void) getchar();     // wait for a char from the keyboard
  
}

// --------------------------------------------------------------------------
// simple while loop delay
void delay(int count){
  while(count){
    --count;
  }
}

// --------------------------------------------------------------------------
//  Delay used between pin High/Low toggles
//  returns 0 if NO key pressed (user interrupted)
//  otherwise returns the key pressed ASCII code
// --------------------------------------------------------------------------
char TogDelay(){

  int count=10000;
  while(count){
    if(kbhit()){
      return getchar();
    }
    --count;
  }
  return 0;
}
// --------------------------------------------------------------------------
//  Delay used for LED blinking
// --------------------------------------------------------------------------
char LEDDelay(){ 
  uint32_t count, count2;

  for(count=50; count; count--){
    for(count2=0x3000; count2; count2--){
      if(kbhit()){
        return getchar();
      }
    }
  }
  return 0;
}

// --------------------------------------------------------------------------
// Disable the software watchdog timer
void DISABLE_WATCHDOG(){
  SWT_0.SR.R = 0x0000c520; // key 
  SWT_0.SR.R = 0x0000d928; // key 
  SWT_0.CR.R = 0x8000010A; // disable WEN 
}                                                                                                                                                                                                                                                                   

// --------------------------------------------------------------------------
void EBIWriteTest(){
  int x;
  unsigned long int *mem_ptr;

  printf("\nWriting 55555555 from %08x - %08x", &EXT_SRAM, EXT_SRAM_START + (EXT_SRAM_SIZE-1)); 
  
  mem_ptr = &EXT_SRAM;
  for (x = 0; x < EXT_SRAM_WORDS; x++){
	  *mem_ptr = 0x55555555;
	  ++mem_ptr;
  }

}


// --------------------------------------------------------------------------
void DDRTest(){

  int x;
  vuint32_t *mem_ptr;
  vuint16_t	*mem_ptr16;
  unsigned long int rval;

  error = 0;

  printf("\n\nDDR - Internal DRAM:");

  printf("\nInitializing .."); 
  SetBusMode(DDR_BUSMODE);	// Set bus to DDR mode

  printf("\nTesting alternating 55555555|AAAAAAAA .."); 
 
  printf("\n   Writing: %08x - %08x", &EXT_DRAM, EXT_DRAM_START + (EXT_DRAM_SIZE-1)); 
  
  // write then read entire array
  mem_ptr = &EXT_DRAM;
  for (x = 0; x < EXT_DRAM_WORDS-1; x+=2){
	  *mem_ptr = 0x55555555;
	  ++mem_ptr;
	  *mem_ptr = 0xAAAAAAAA;
	  ++mem_ptr;
  }

  printf("\n   Reading: %08x - %08x", &EXT_DRAM, EXT_DRAM_START + (EXT_DRAM_SIZE-1)); 
  mem_ptr = &EXT_DRAM;
  for (x = 0; x < EXT_DRAM_WORDS-1; x+=2){
	  rval = *mem_ptr;
	  if (rval != 0x55555555){
	    ++error;
	    printf("\naddr: %08x    wrote %08x  read %08x", mem_ptr, 0x55555555, rval); 
	  }
  	++mem_ptr;

	  rval = *mem_ptr;
	  if (rval != 0xAAAAAAAA){
	    ++error;
	    printf("\naddr: %08x    wrote %08x  read %08x", mem_ptr, 0xAAAAAAAA, rval); 
	  }
  	++mem_ptr;

  	if(error > 8){
	    printf("\nAborting test.");
  	  break;  // abort if too many errors
  	}
  }
  
  printf("\nTesting all 00000000.."); 
  mem_ptr = &EXT_DRAM;
  for (x = 0; x < EXT_DRAM_WORDS-1; x+=2){
	  *mem_ptr = 0x00000000;
	  rval = *mem_ptr;
	  if (rval != 0x00000000){
	    ++error;
	    printf("\naddr: %08x    wrote %08x  read %08x", mem_ptr, 0x00000000, rval); 
	  }
  	++mem_ptr;
  }

  printf("\nTesting all FFFFFFFF.."); 
  mem_ptr = &EXT_DRAM;
  for (x = 0; x < EXT_DRAM_WORDS-1; x+=2){
	  *mem_ptr = 0xFFFFFFFF;
	  rval = *mem_ptr;
	  if (rval != 0xFFFFFFFF){
	    ++error;
	    printf("\naddr: %08x    wrote %08x  read %08x", mem_ptr, 0xFFFFFFFF, rval); 
	  }
  	++mem_ptr;
  }
  
  printf("\nTesting 16-bit.."); 
  mem_ptr16 = &EXT_DRAM16;
  *mem_ptr16 = 0x8420;
  if(*mem_ptr16 != 0x8420){
    printf("\naddr: %08x    wrote %04x  read %04x", mem_ptr16, 0x8420, *mem_ptr16); 
  }
  
	if(error)  printf("\n\n--- DDR Memory Test:  %d  Errors\n", error);
	else			 printf("\nPASSED\n");
}


// --------------------------------------------------------------------------
// CLKOUT() - Outputs chosen sysclk
void CLKOUT(){
    /* Configure CLKOUT on PB6 */
    //SIU.PCR[22].R      = 0x0600;      		// PB3 clkout (0x0A00)

    /* Choose Clock source to be output */ 
//    CGM.OCDS_SC.B.SELCTL = 0x0; 	   	 	// 16MHz IRC Osc is Output Clock (Internal)
//    CGM.OCDS_SC.B.SELCTL = 0x1; 	   	 	// 4MHz crystal Osc is Output Clock (External crystal 40Mhz on my EVB)
    CGM.OCDS_SC.B.SELCTL = 0x2; 	   	 	// PLL0 is Output Clock (180MHz)
//    CGM.OCDS_SC.B.SELCTL = 0x3; 	   	 	// PLL1 is Output Clock (180MHz)

    /* Choose what to divide output by */
//    CGM.OCDS_SC.B.SELDIV = 0x0; 	   	 	// Divide Output Clock by 1 (No Division)
//    CGM.OCDS_SC.B.SELDIV = 0x1; 	   	 	// Divide Output Clock by 2
#ifdef CLOCK_SPEED_90MHZ
    CGM.OCDS_SC.B.SELDIV = 0x2; 	   	 	// Divide Output Clock by 4
#endif
#ifdef CLOCK_SPEED_180MHZ
    CGM.OCDS_SC.B.SELDIV = 0x3; 	   	 	// Divide Output Clock by 8
#endif

    /* Enable clock output */
    CGM.OC_EN.B.EN       = 0x1; 	   	  	// Enable Output Clock
}

 
// --------------------------------------------------------------------------
void setup_fmpll(){

// Enable Clockout at Div 8 
// SIU.PCR[22].R      = 0x0600;        // Configure CLKOUT on B3 	

// Select 1 from below          
//    CGM.OCDSSC.B.SELCTL = 3; 	   	 	// PLL1 is Output Clock
    CGM.OCDS_SC.B.SELCTL = 2; 	   	 	// PLL0 is Output Clock 
//    CGM.OCDSSC.B.SELCTL = 1; 	   	 	// OSC is Output Clock
//    CGM.OCDS_SC.B.SELCTL = 0; 	   	 	// IRC is Output Clock 
    CGM.OCDS_SC.B.SELDIV = 3; 	   	 	// Divide Output Clock by 8
    CGM.OC_EN.B.EN       = 1; 	   	  	// Enable Output Clock

 ME.MER.R = 0x000025FF;          // Enable all modes                     		
 ME.RUNPC[0].R = 0x000000FE;     // Enable all peripherals in all modes  
 
 //ME.RESET.B.XOSCON = 1;			   // Switch on external osc
 ME.DRUN.B.XOSCON = 1;			   // Switch on external osc 

// CGM.OCDS_SC.B.SELDIV = 3;
    
 // RE  enter the drun mode, to update the configuration 
 ME.MCTL.R = 0x30005AF0;                // Mode & Key 
 ME.MCTL.R = 0x3000A50F;                // Mode & Key inverted 

 while(ME.GS.B.S_MTRANS == 1){;}          // Wait for mode entry to complete 
 while(ME.GS.B.S_CURRENTMODE != 3){;}     // Check DRUN mode has been entered 

 while(!ME.GS.B.S_XOSC){;}		 // Wait for clock to stabilise 

 // Selecting XOSC as the FMPLL_0 Source 
 CGM.AC3_SC.B.SELCTL = 0x1;
  
 // For XTAL = 40MHz, VCO = 256M, PLL OUT = 16M 
 CGM.FMPLL[0].CR.B.IDF = 0x7;		// Divide by 8 
 CGM.FMPLL[0].CR.B.ODF = 0x0;		// Divide by 2 
 CGM.FMPLL[0].CR.B.NDIV = CLOCK_NDIV;		
 
 ME.DRUN.B.PLL0ON  = 1;

// RE enter the drun mode, to update the configuration 
 ME.MCTL.R = 0x30005AF0;     	        // Mode & Key 
 ME.MCTL.R = 0x3000A50F;                // Mode & Key inverted 

 while(ME.GS.B.S_MTRANS == 1){;}		// Wait for mode entry to complete 
 while(ME.GS.B.S_CURRENTMODE != 0x3){;}   // Check DRUN mode has been entered 

// wait for PLL to lock - will not lock before DRUN re-entry 
 while(CGM.FMPLL[0].CR.B.S_LOCK == 0){;}

 CGM.OCDS_SC.B.SELCTL = 2;		 // Select Output clock System PLL_0 

// Set the sysclock as PLL0 
 ME.DRUN.B.SYSCLK = 4; 
 
// RE enter the drun mode, to update the configuration 
 ME.MCTL.R = 0x30005AF0;     	        // Mode & Key 
 ME.MCTL.R = 0x3000A50F;                // Mode & Key inverted 

 while(ME.GS.B.S_MTRANS == 1){;}				// Wait for mode entry to complete 
 while(ME.GS.B.S_CURRENTMODE != 0x3){;}   // Check DRUN mode has been entered 
 
} 


#define SLEW_HALF	0x0000
#define SLEW_FULL	0x0004

#define PCR_OUT	0x0200
#define PCR_IN	0x0100

#define PCR_A1	0x0400
#define PCR_A2	0x0800
#define PCR_A3	0x0C00

#define PCR_ODE	0x0020


//#define _SCR 0x0A8042C0 //dual	//RS=5; BS=4; 16BIT=0; RDLY=2; HALF=0; QURT=0; WDLY=2; EARLY=1; ODT=1
//#define _SCR 0x040042C0 
//#define _SCR 0x04004200 // removed OD
//#define _SCR 0x04005200 // extra half clock to read dly
#define _SCR 0x04004100 // extra clock to write dly
//#define _SCR 0x062042C0 // test #1
//#define _SCR 0x086042C0 // test #2
//#define _SCR 0x0AA042C0 // test #3

// Micron LPDDR SDRAM Commands
#define MIC_CS	0x00400000	// bit 22 CS#
#define MIC_RAS	0x00200000	// bit 21 RAS
#define MIC_CAS	0x00100000	// bit 20 CAS
#define MIC_WE	0x00080000	// bit 19 WE
#define MIC_BA2	0x00040000	// bit 18 BA2
#define MIC_BA1	0x00020000	// bit 17 BA1
#define MIC_BA0	0x00010000	// bit 16 BA0
// 14-0 = A14-A0

void Delay_NOP(unsigned long int val){
	while(val--){
		MDDRC.DDR_COMMAND.R = 0x00380000; // Issue NOP
	}
}

// --------------------------------------------------------------------------
void SetBusMode(char mode){

  vuint32_t *RegPtr;
  vuint16_t	val, i;

  val = 0x0100;		// make all bus ports INPUT, no pullups
  for(i=147;  i<=156; i++)  	SIU.PCR[i].R = val;
  for(i=158;  i<=197; i++)  	SIU.PCR[i].R = val;

	// --- DDR Mode
	if(mode){

		// disable EBI
	  RegPtr = (vuint32_t *) EBI_BASE_REG;  // MCR
  	*RegPtr = 0x00000010; // MDIS - module disabled
		
	// enable DDR: GPIO[129] = DDR ON	active high
    SIU.GPDO[BUS_CONTROL_LINE1].B.PDO = 1;  	// make pins high, bus = 1.8V
    SIU.GPDO[BUS_CONTROL_LINE2].B.PDO = 1;  

	// wait a microsecond before returning
	delay(100);	// ?? increase or decrease this?

  // set PCR registers used by DDR
  SIU.PCR[147].R = (PCR_A1 + PCR_OUT + SLEW_FULL);	// alt mode 1
  SIU.PCR[148].R = PCR_IN;	// gpio input
  SIU.PCR[149].R = PCR_IN;	// gpio input
  for(i=150; i<=173; i++)  	SIU.PCR[i].R = (PCR_A1 + PCR_OUT + SLEW_FULL);	// alt mode 1
  for(i=174; i<=193; i++)  	SIU.PCR[i].R = (PCR_A1 + PCR_OUT + PCR_IN + SLEW_FULL);	// alt mode 1


	MDDRC.DDR_SYS_CONFIG.R = 0xB0000000;	// Enable clock = no CKE

	// Configure DRAM Control Registers
	MDDRC.DDR_SYS_CONFIG.R = (0xB0000000 + _SCR);

	MDDRC.DDR_TIME_CONFIG0.R = 0x03C7261C;	//REFRESH=$3C7; CMD=$26; PRE=$1C
	MDDRC.DDR_TIME_CONFIG1.R = 0x40CC10E5;	//RFC=$10; WR1=$06; WTR1=$06; RRD=$02; RC=$07; RAS=$05
	MDDRC.DDR_TIME_CONFIG2.R = 0x28321042;	

	delay(10000);	// wait c. 200us

	MDDRC.DDR_COMMAND.R = MIC_CS; // CS#
	delay(5);	

	// Set CKE high
	MDDRC.DDR_SYS_CONFIG.R = (0xF0000000 + _SCR);		// Enable CKE
	delay(20);	// Wait for 400ns
	Delay_NOP(1000);	// NOP operations for 200us


	// Send commands directly to DRAM

	// 0-14 = A0 - A14
	// 15 = 0
	// 16-18 = BA0 - BA2
	// 19 = WEB
	// 20 = CAS
	// 21 = RAS
	// 22 = CS0

		// precharge all command
		MDDRC.DDR_COMMAND.R = 0x00100400; // Issue PreCharge All
		Delay_NOP(5);	

		// refresh command 
		MDDRC.DDR_COMMAND.R = 0x00080000; // Issue Refresh
		Delay_NOP(5);	

		// refresh command
		MDDRC.DDR_COMMAND.R = 0x00080000; // Issue Refresh
		Delay_NOP(5);	

		// standard mode register command 
		MDDRC.DDR_COMMAND.R = 0x00000032; 
		Delay_NOP(5);	

		// extended mode register command 
		MDDRC.DDR_COMMAND.R = (0x00000000 + MIC_BA1); 	// full drive
		Delay_NOP(5);	

		MDDRC.DDR_SYS_CONFIG.R = (0xE0000000 + _SCR);	//End Command
		
		delay(100);	// ?? increase or decrease this?
	}

	// --- EBI Mode
	else{
	  // set PCR registers used by EBI 
	  for(i=147;  i<=154; i++)  	SIU.PCR[i].R = PCR_A2 + PCR_OUT + SLEW_FULL;	// alt mode 2, output
	  SIU.PCR[155].R = PCR_OUT;						// gpio output
	  SIU.PCR[156].R = PCR_A2 + PCR_OUT + SLEW_FULL;	// alt mode 2, output
    // no 157 
	  SIU.PCR[158].R = PCR_A3 + PCR_OUT;	// alt mode 3, output
	  SIU.PCR[159].R = PCR_IN;						// gpio input
	  SIU.PCR[160].R = PCR_A3 + PCR_IN + 0x03;	// (TA) 
	  SIU.PCR[161].R = PCR_A3 + PCR_IN + 0x03;	// (TEA)

	  for(i=162;  i<=165; i++)  	SIU.PCR[i].R = PCR_IN;	// gpio input
	  for(i=166;  i<=197; i++)  	SIU.PCR[i].R = PCR_A2 + PCR_OUT + PCR_IN + SLEW_FULL;	// alt mode 2, input & output

		// GPIO[128] = EBI ON	active low
    SIU.GPDO[BUS_CONTROL_LINE1].B.PDO = 0;  	// make pins low, bus = 3V
    SIU.GPDO[BUS_CONTROL_LINE2].B.PDO = 0;  

		// wait a microsecond before returning
		delay(100);	// ?? increase or decrease this?

		// setup EBI registers
  	RegPtr = (vuint32_t *) EBI_BASE_REG;  // MCR
  	*RegPtr = 0x00000006; // NON CS access = 32-bit muxed mode, D16-D31 = 16bit 
	}
}

// --------------------------------------------------------------------------
// Setup EBI
void setup_EBI(){
  vuint32_t *RegPtr;

  SIU.GPDO[BUS_CONTROL_LINE1].B.PDO = 0;  	// make pins low, bus = 3V
  SIU.GPDO[BUS_CONTROL_LINE2].B.PDO = 0;  

	// make bus mode pins GPIO outputs
	SIU.PCR[BUS_CONTROL_LINE1].R = 0x0200;	
	SIU.PCR[BUS_CONTROL_LINE2].R = 0x0200;	

	SetBusMode(EBI_BUSMODE);	// Set bus to EBI mode

  // config the base register: Address 2000 0000
  RegPtr = (vuint32_t *) (EBI_BASE_REG + 0x14);  	// bank 0
  *RegPtr = 0xFF800010; // OR0    1 wait
//  *RegPtr = 0xFF800000; // OR0      0 wait
  RegPtr = (vuint32_t *) (EBI_BASE_REG + 0x10);  	// bank 0
//  *RegPtr = 0x200000AF; // BR0    //--- CONFIGURE CS0 - 20000000 - 207FFFFF
//  *RegPtr = 0x200000AB; // BR0    //--- CONFIGURE CS0 - 20000000 - 207FFFFF
  *RegPtr = 0x200010AB; // BR0    //--- CONFIGURE CS0 - 20000000 - 207FFFFF

  RegPtr = (vuint32_t *) (EBI_BASE_REG + 0x24);  	// bank 2
  *RegPtr++ = 0xFF8000F0; // OR2    
  RegPtr = (vuint32_t *) (EBI_BASE_REG + 0x20);  	// bank 2
  *RegPtr++ = 0x208008C3; // BR2    //--- CONFIGURE CS2 - 20800000 - 20FFFFFF

  *RegPtr-- = 0xFF8000F0;   // OR1
  *RegPtr = 0x208008C3; // BR1    
}

// --------------------------------------------------------------------------
void initialize_hardware(){
  vuint16_t	val, i;
  
  val = 0x0103;	// make INPUT, push pull by default

	// set default GPIO for all pins
  for(i=0;  i<=34; i++)  	SIU.PCR[i].R = val;
  for(i=36; i<=39; i++)  	SIU.PCR[i].R = val;
  for(i=42; i<=60; i++)  	SIU.PCR[i].R = val;
  SIU.PCR[62].R = val;
  SIU.PCR[64].R = val;
  SIU.PCR[66].R = val;
  for(i=68; i<=71; i++)  	SIU.PCR[i].R = val;
  for(i=73; i<=76; i++)  	SIU.PCR[i].R = val;
  SIU.PCR[78].R = val;
  SIU.PCR[80].R = val;
  for(i=84; i<=95; i++)  	SIU.PCR[i].R = val;
  for(i=98; i<=103; i++) 	SIU.PCR[i].R = val;
  for(i=108; i<=146; i++)	SIU.PCR[i].R = val;
	// between these used by data bus
  for(i=198; i<=233; i++)	SIU.PCR[i].R = val;

  // configure bus mode control lines
  SIU.GPDO[BUS_CONTROL_LINE1].B.PDO = 0;  	// make pin low (was 128)
  SIU.GPDO[BUS_CONTROL_LINE2].B.PDO = 0;  	// make pin low (was 129)
  SIU.PCR[BUS_CONTROL_LINE1].R = 0x0200;	    // gpio output
  SIU.PCR[BUS_CONTROL_LINE2].R = 0x0200;	    // gpio output
  
//	MC_MODE_INIT();		// set magic carpet 

	setup_fmpll();	// Setup PLL 
//	SET_PLL_0();		// Set PLL 0
//	SET_PLL_1();		// Set PLL 1
	CLKOUT();		// Output Selected Clock Source 
	
//	SIU.PCR[148].R = 0x0220;  //FLEXPWM0_B0 Ball=K17
	
	sci_a_init();	// initialize SCI_A serial port (1st core uses SCI_A)


	setup_EBI();	// initialize the external bus interface and chip selects
  
}
